@@ -34,11 +34,17 @@
 
 #include <string.h>
 #include <unistd.h>
+#include <stdlib.h>
+#include <locale.h>
 
 #include <glib/gi18n.h>
 #include <libpeas/peas-extension-set.h>
 #include <gtksourceview/gtksourcestyleschememanager.h>
 
+#ifdef ENABLE_INTROSPECTION
+#include <girepository.h>
+#endif
+
 #include "gedit-app.h"
 #include "gedit-commands.h"
 #include "gedit-notebook.h"
@@ -50,14 +56,9 @@
 #include "gedit-app-activatable.h"
 #include "gedit-plugins-engine.h"
 
-#ifdef OS_OSX
-#include "gedit-app-osx.h"
-#else
-#ifdef G_OS_WIN32
-#include "gedit-app-win32.h"
-#else
-#include "gedit-app-x11.h"
-#endif
+#ifndef ENABLE_GVFS_METADATA
+#include "gedit-metadata-manager.h"
+#define METADATA_FILE "gedit-metadata.xml"
 #endif
 
 #define GEDIT_PAGE_SETUP_FILE		"gedit-page-setup"
@@ -74,8 +75,7 @@ enum
 
 struct _GeditAppPrivate
 {
-	GList	          *windows;
-	GeditWindow       *active_window;
+	GeditPluginsEngine *engine;
 
 	GeditLockdownMask  lockdown;
 
@@ -88,19 +88,104 @@ struct _GeditAppPrivate
 	PeasExtensionSet  *extensions;
 };
 
-static GeditApp *app_instance = NULL;
+static gboolean help = FALSE;
+static gboolean version = FALSE;
+static gboolean list_encodings = FALSE;
+static gchar *encoding_charset = NULL;
+static gboolean new_window = FALSE;
+static gboolean new_document = FALSE;
+static gchar *geometry = NULL;
+static gboolean wait = FALSE;
+static gboolean standalone = FALSE;
+static gchar **remaining_args = NULL;
+static const GeditEncoding *encoding = NULL;
+static GInputStream *stdin_stream = NULL;
+static GSList *file_list = NULL;
+static gint line_position = 0;
+static gint column_position = 0;
+static GApplicationCommandLine *command_line = NULL;
 
-G_DEFINE_ABSTRACT_TYPE(GeditApp, gedit_app, G_TYPE_OBJECT)
-
-static void
-gedit_app_finalize (GObject *object)
+static const GOptionEntry options[] =
 {
-	GeditApp *app = GEDIT_APP (object); 
+	/* Help */
+	{
+		"help", '?', 0, G_OPTION_ARG_NONE, &help,
+		N_("Show the application's help"), NULL
+	},
 
-	g_list_free (app->priv->windows);
+	/* Version */
+	{
+		"version", 'V', 0, G_OPTION_ARG_NONE, &version,
+		N_("Show the application's version"), NULL
+	},
 
-	G_OBJECT_CLASS (gedit_app_parent_class)->finalize (object);
-}
+	/* List available encodings */
+	{
+		"list-encodings", '\0', 0, G_OPTION_ARG_NONE, &list_encodings,
+		N_("Display list of possible values for the encoding option"),
+		NULL
+	},
+
+	/* Encoding */
+	{
+		"encoding", '\0', 0, G_OPTION_ARG_STRING,
+		&encoding_charset,
+		N_("Set the character encoding to be used to open the files listed on the command line"),
+		N_("ENCODING")
+	},
+
+	/* Open a new window */
+	{
+		"new-window", '\0', 0, G_OPTION_ARG_NONE,
+		&new_window,
+		N_("Create a new top-level window in an existing instance of gedit"),
+		NULL
+	},
+
+	/* Create a new empty document */
+	{
+		"new-document", '\0', 0, G_OPTION_ARG_NONE,
+		&new_document,
+		N_("Create a new document in an existing instance of gedit"),
+		NULL
+	},
+
+	/* Window geometry */
+	{
+		"geometry", 'g', 0, G_OPTION_ARG_STRING,
+		&geometry,
+		N_("Set the size and position of the window (WIDTHxHEIGHT+X+Y)"),
+		N_("GEOMETRY")
+	},
+
+	/* Wait for closing documents */
+	{
+		"wait", 'w', 0, G_OPTION_ARG_NONE,
+		&wait,
+		N_("Open files and block process until files are closed"),
+		NULL
+	},
+
+	/* New instance */
+	{
+		"standalone", 's', 0, G_OPTION_ARG_NONE,
+		&standalone,
+		N_("Run gedit in standalone mode"),
+		NULL
+	},
+
+	/* collects file arguments */
+	{
+		G_OPTION_REMAINING, '\0', 0, G_OPTION_ARG_FILENAME_ARRAY,
+		&remaining_args,
+		NULL,
+		N_("[FILE...] [+LINE[:COLUMN]]")
+	},
+
+	{NULL}
+};
+
+G_DEFINE_ABSTRACT_TYPE(GeditApp, gedit_app, GTK_TYPE_APPLICATION)
 
 static void
 gedit_app_dispose (GObject *object)
@@ -117,6 +202,8 @@ gedit_app_dispose (GObject *object)
 	   all extensions which in turn will deactivate the extension */
 	g_clear_object (&app->priv->extensions);
 
+	g_clear_object (&app->priv->engine);
+
 	G_OBJECT_CLASS (gedit_app_parent_class)->dispose (object);
 }
 
@@ -139,34 +226,6 @@ gedit_app_get_property (GObject    *object,
 	}
 }
 
-static GObject *
-gedit_app_constructor (GType                  gtype,
-                       guint                  n_construct_params,
-                       GObjectConstructParam *construct_params)
-{
-	if (!app_instance)
-	{
-		GObject *obj = G_OBJECT_CLASS (
-			gedit_app_parent_class)->constructor (gtype,
-			                                      n_construct_params,
-			                                      construct_params);
-
-		app_instance = GEDIT_APP (obj);
-		g_object_add_weak_pointer (obj, (gpointer *) &app_instance);
-
-		return obj;
-	}
-
-	return g_object_ref (app_instance);
-}
-
-static gboolean
-gedit_app_last_window_destroyed_impl (GeditApp    *app,
-                                      GeditWindow *window)
-{
-	return TRUE;
-}
-
 static gchar *
 gedit_app_help_link_id_impl (GeditApp    *app,
                              const gchar *name,
@@ -247,6 +306,528 @@ gedit_app_set_window_title_impl (GeditApp    *app,
 	gtk_window_set_title (GTK_WINDOW (window), title);
 }
 
+static void
+extension_added (PeasExtensionSet *extensions,
+		 PeasPluginInfo   *info,
+		 PeasExtension    *exten,
+		 GeditApp         *app)
+{
+	gedit_app_activatable_activate (GEDIT_APP_ACTIVATABLE (exten));
+}
+
+static void
+extension_removed (PeasExtensionSet *extensions,
+		   PeasPluginInfo   *info,
+		   PeasExtension    *exten,
+		   GeditApp         *app)
+{
+	gedit_app_activatable_deactivate (GEDIT_APP_ACTIVATABLE (exten));
+}
+
+static void
+gedit_app_startup (GApplication *application)
+{
+	GeditApp *app = GEDIT_APP (application);
+	GtkSourceStyleSchemeManager *manager;
+	const gchar *dir;
+	gchar *icon_dir;
+#ifndef ENABLE_GVFS_METADATA
+	const gchar *cache_dir;
+	gchar *metadata_filename;
+#endif
+
+	G_APPLICATION_CLASS (gedit_app_parent_class)->startup (application);
+
+	/* Setup debugging */
+	gedit_debug_init ();
+	gedit_debug_message (DEBUG_APP, "Startup");
+
+	/* Setup locale/gettext */
+	setlocale (LC_ALL, "");
+
+	dir = gedit_dirs_get_gedit_locale_dir ();
+	bindtextdomain (GETTEXT_PACKAGE, dir);
+
+	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
+	textdomain (GETTEXT_PACKAGE);
+
+	gedit_dirs_init ();
+
+	gedit_debug_message (DEBUG_APP, "Set icon");
+
+	dir = gedit_dirs_get_gedit_data_dir ();
+	icon_dir = g_build_filename (dir, "icons", NULL);
+
+	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (), icon_dir);
+	g_free (icon_dir);
+
+#ifndef ENABLE_GVFS_METADATA
+	/* Setup metadata-manager */
+	cache_dir = gedit_dirs_get_user_cache_dir ();
+
+	metadata_filename = g_build_filename (cache_dir, METADATA_FILE, NULL);
+
+	gedit_metadata_manager_init (metadata_filename);
+
+	g_free (metadata_filename);
+#endif
+
+	/* Load settings */
+	app->priv->settings = gedit_settings_new ();
+	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
+
+	/* initial lockdown state */
+	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
+
+	/*
+	 * We use the default gtksourceview style scheme manager so that plugins
+	 * can obtain it easily without a gedit specific api, but we need to
+	 * add our search path at startup before the manager is actually used.
+	 */
+	manager = gtk_source_style_scheme_manager_get_default ();
+	gtk_source_style_scheme_manager_append_search_path (manager,
+	                                                    gedit_dirs_get_user_styles_dir ());
+
+	app->priv->engine = gedit_plugins_engine_get_default ();
+	app->priv->extensions = peas_extension_set_new (PEAS_ENGINE (app->priv->engine),
+	                                                GEDIT_TYPE_APP_ACTIVATABLE,
+	                                                "app", app,
+	                                                NULL);
+
+	g_signal_connect (app->priv->extensions,
+	                  "extension-added",
+	                  G_CALLBACK (extension_added),
+	                  app);
+
+	g_signal_connect (app->priv->extensions,
+	                  "extension-removed",
+	                  G_CALLBACK (extension_removed),
+	                  app);
+
+	peas_extension_set_foreach (app->priv->extensions,
+	                            (PeasExtensionSetForeachFunc) extension_added,
+	                            app);
+}
+
+static gboolean
+is_in_viewport (GtkWindow    *window,
+		GdkScreen    *screen,
+		gint          workspace,
+		gint          viewport_x,
+		gint          viewport_y)
+{
+	GdkScreen *s;
+	GdkDisplay *display;
+	GdkWindow *gdkwindow;
+	const gchar *cur_name;
+	const gchar *name;
+	gint cur_n;
+	gint n;
+	gint ws;
+	gint sc_width, sc_height;
+	gint x, y, width, height;
+	gint vp_x, vp_y;
+
+	/* Check for screen and display match */
+	display = gdk_screen_get_display (screen);
+	cur_name = gdk_display_get_name (display);
+	cur_n = gdk_screen_get_number (screen);
+
+	s = gtk_window_get_screen (window);
+	display = gdk_screen_get_display (s);
+	name = gdk_display_get_name (display);
+	n = gdk_screen_get_number (s);
+
+	if (strcmp (cur_name, name) != 0 || cur_n != n)
+	{
+		return FALSE;
+	}
+
+	/* Check for workspace match */
+	ws = gedit_utils_get_window_workspace (window);
+	if (ws != workspace && ws != GEDIT_ALL_WORKSPACES)
+	{
+		return FALSE;
+	}
+
+	/* Check for viewport match */
+	gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
+	gdk_window_get_position (gdkwindow, &x, &y);
+	width = gdk_window_get_width (gdkwindow);
+	height = gdk_window_get_height (gdkwindow);
+	gedit_utils_get_current_viewport (screen, &vp_x, &vp_y);
+	x += vp_x;
+	y += vp_y;
+
+	sc_width = gdk_screen_get_width (screen);
+	sc_height = gdk_screen_get_height (screen);
+
+	return x + width * .25 >= viewport_x &&
+	       x + width * .75 <= viewport_x + sc_width &&
+	       y >= viewport_y &&
+	       y + height <= viewport_y + sc_height;
+}
+
+static GeditWindow *
+get_active_window (GtkApplication *app)
+{
+	GdkScreen *screen;
+	guint workspace;
+	gint viewport_x, viewport_y;
+	GList *windows, *l;
+
+	screen = gdk_screen_get_default ();
+
+	workspace = gedit_utils_get_current_workspace (screen);
+	gedit_utils_get_current_viewport (screen, &viewport_x, &viewport_y);
+
+	/* Gtk documentation says the window list is always in MRU order */
+	windows = gtk_application_get_windows (app);
+	for (l = windows; l != NULL; l = l->next)
+	{
+		GtkWindow *window = l->data;
+
+		if (is_in_viewport (window, screen, workspace, viewport_x, viewport_y))
+		{
+			return GEDIT_WINDOW (window);
+		}
+	}
+
+	return NULL;
+}
+
+static void
+set_command_line_wait (GeditTab *tab)
+{
+	g_object_set_data_full (G_OBJECT (tab),
+	                        "GeditTabCommandLineWait",
+	                        g_object_ref (command_line),
+	                        (GDestroyNotify)g_object_unref);
+}
+
+static void
+gedit_app_activate (GApplication *application)
+{
+	GeditWindow *window = NULL;
+	GeditTab *tab;
+	gboolean doc_created = FALSE;
+
+	if (!new_window)
+	{
+		window = get_active_window (GTK_APPLICATION (application));
+	}
+
+	if (window == NULL)
+	{
+		gedit_debug_message (DEBUG_APP, "Create main window");
+		window = gedit_app_create_window (GEDIT_APP (application), NULL);
+
+		gedit_debug_message (DEBUG_APP, "Show window");
+		gtk_widget_show (GTK_WIDGET (window));
+	}
+
+	if (geometry)
+	{
+		gtk_window_parse_geometry (GTK_WINDOW (window),
+		                           geometry);
+	}
+
+	if (stdin_stream)
+	{
+		gedit_debug_message (DEBUG_APP, "Load stdin");
+
+		tab = gedit_window_create_tab_from_stream (window,
+		                                           stdin_stream,
+		                                           encoding,
+		                                           line_position,
+		                                           column_position,
+		                                           TRUE);
+		doc_created = tab != NULL;
+
+		if (doc_created && command_line)
+		{
+			set_command_line_wait (tab);
+		}
+		g_input_stream_close (stdin_stream, NULL, NULL);
+	}
+
+	if (file_list != NULL)
+	{
+		GSList *loaded;
+
+		gedit_debug_message (DEBUG_APP, "Load files");
+		loaded = _gedit_cmd_load_files_from_prompt (window,
+		                                            file_list,
+		                                            encoding,
+		                                            line_position,
+		                                            column_position);
+
+		doc_created = doc_created || loaded != NULL;
+
+		if (command_line)
+		{
+			g_slist_foreach (loaded, (GFunc)set_command_line_wait, NULL);
+		}
+		g_slist_free (loaded);
+	}
+
+	if (!doc_created || new_document)
+	{
+		gedit_debug_message (DEBUG_APP, "Create tab");
+		tab = gedit_window_create_tab (window, TRUE);
+
+		if (command_line)
+		{
+			set_command_line_wait (tab);
+		}
+	}
+
+	gtk_window_present (GTK_WINDOW (window));
+}
+
+static GOptionContext *
+get_option_context (void)
+{
+	GOptionContext *context;
+
+	context = g_option_context_new (_("- Edit text files"));
+	g_option_context_add_main_entries (context, options, GETTEXT_PACKAGE);
+	g_option_context_add_group (context, gtk_get_option_group (FALSE));
+
+#ifdef ENABLE_INTROSPECTION
+	g_option_context_add_group (context, g_irepository_get_option_group ());
+#endif
+
+	return context;
+}
+
+static gboolean
+option_context_parse (GOptionContext  *context,
+                      gchar          **arguments,
+                      GError         **error)
+{
+	gint argc;
+	gchar **argv;
+	gint i;
+	gboolean ret;
+
+	/* We have to make an extra copy of the array, since g_option_context_parse()
+	 * assumes that it can remove strings from the array without freeing them.
+	 */
+	argc = g_strv_length (arguments);
+	argv = g_new (gchar *, argc);
+	for (i = 0; i < argc; i++)
+	{
+		argv[i] = arguments[i];
+	}
+
+	ret = g_option_context_parse (context, &argc, &argv, error);
+
+	g_free (argv);
+
+	return ret;
+}
+
+static void
+clear_options (void)
+{
+	g_free (encoding_charset);
+	g_strfreev (remaining_args);
+	g_free (geometry);
+	g_clear_object (&stdin_stream);
+	g_slist_free_full (file_list, g_object_unref);
+
+	help = FALSE;
+	version = FALSE;
+	list_encodings = FALSE;
+	encoding_charset = NULL;
+	new_window = FALSE;
+	new_document = FALSE;
+	geometry = NULL;
+	wait = FALSE;
+	standalone = FALSE;
+	remaining_args = NULL;
+	encoding = NULL;
+	file_list = NULL;
+	line_position = 0;
+	column_position = 0;
+	command_line = NULL;
+}
+
+static void
+get_line_column_position (const gchar *arg,
+                          gint        *line,
+                          gint        *column)
+{
+	gchar **split;
+
+	split = g_strsplit (arg, ":", 2);
+
+	if (split != NULL)
+	{
+		if (split[0] != NULL)
+		{
+			*line = atoi (split[0]);
+		}
+
+		if (split[1] != NULL)
+		{
+			*column = atoi (split[1]);
+		}
+	}
+
+	g_strfreev (split);
+}
+
+static gint
+gedit_app_command_line (GApplication            *application,
+                        GApplicationCommandLine *cl)
+{
+	gchar **arguments;
+	GOptionContext *context;
+	GError *error = NULL;
+
+	arguments = g_application_command_line_get_arguments (cl, NULL);
+
+	context = get_option_context ();
+
+	/* Avoid exit() on the main instance */
+	g_option_context_set_help_enabled (context, FALSE);
+
+	if (!option_context_parse (context, arguments, &error))
+	{
+		/* We should never get here since parsing would have
+		 * failed on the client side... */
+		g_application_command_line_printerr (cl,
+		                                     _("%s\nRun '%s --help' to see a full list of available command line options.\n"),
+		                                     error->message, arguments[0]);
+
+		g_error_free (error);
+		g_application_command_line_set_exit_status (cl, 1);
+	}
+	else
+	{
+		if (wait)
+		{
+			command_line = cl;
+		}
+
+		/* Parse encoding */
+		if (encoding_charset)
+		{
+			encoding = gedit_encoding_get_from_charset (encoding_charset);
+
+			if (encoding == NULL)
+			{
+				g_application_command_line_printerr (cl,
+				                                     _("%s: invalid encoding."),
+				                                     encoding_charset);
+			}
+
+			g_free (encoding_charset);
+		}
+
+		/* Parse filenames */
+		if (remaining_args)
+		{
+			gint i;
+
+			for (i = 0; remaining_args[i]; i++)
+			{
+				if (*remaining_args[i] == '+')
+				{
+					if (*(remaining_args[i] + 1) == '\0')
+					{
+						/* goto the last line of the document */
+						line_position = G_MAXINT;
+						column_position = 0;
+					}
+					else
+					{
+						get_line_column_position (remaining_args[i] + 1,
+							                  &line_position,
+							                  &column_position);
+					}
+				}
+				else if (*remaining_args[i] == '-' && *(remaining_args[i] + 1) == '\0')
+				{
+					stdin_stream = g_application_command_line_get_stdin (cl);
+				}
+				else
+				{
+					GFile *file;
+
+					file = g_application_command_line_create_file_for_arg (cl, remaining_args[i]);
+					file_list = g_slist_prepend (file_list, file);
+				}
+			}
+
+			file_list = g_slist_reverse (file_list);
+		}
+
+		g_application_activate (application);
+	}
+
+	g_option_context_free (context);
+	clear_options ();
+
+	return 0;
+}
+
+static gboolean
+gedit_app_local_command_line (GApplication   *application,
+                              gchar        ***arguments,
+                              gint           *exit_status)
+{
+	GOptionContext *context;
+	GError *error = NULL;
+	gboolean ret = FALSE;
+
+	/* Handle some of the option without contacting the main instance */
+	context = get_option_context ();
+
+	if (!option_context_parse (context, *arguments, &error))
+	{
+		g_printerr (_("%s\nRun '%s --help' to see a full list of available command line options.\n"),
+		           error->message, (*arguments)[0]);
+
+		g_error_free (error);
+		*exit_status = 1;
+		ret = TRUE;
+	}
+	else if (version)
+	{
+		g_print ("%s - Version %s\n", g_get_application_name (), VERSION);
+		ret = TRUE;
+	}
+	else if (list_encodings)
+	{
+		gint i = 0;
+		const GeditEncoding *enc;
+
+		while ((enc = gedit_encoding_get_from_index (i)) != NULL)
+		{
+			g_print ("%s\n", gedit_encoding_get_charset (enc));
+
+			++i;
+		}
+
+		ret = TRUE;
+	}
+	else if (standalone)
+	{
+		GApplicationFlags old_flags;
+
+		old_flags = g_application_get_flags (application);
+		g_application_set_flags (application, old_flags | G_APPLICATION_NON_UNIQUE);
+	}
+
+	g_option_context_free (context);
+	clear_options ();
+
+	return ret ? ret : G_APPLICATION_CLASS (gedit_app_parent_class)->local_command_line (application, arguments, exit_status);
+}
+
 static gboolean
 ensure_user_config_dir (void)
 {
@@ -370,7 +951,7 @@ save_print_settings (GeditApp *app)
 }
 
 static void
-gedit_app_quit_impl (GeditApp *app)
+gedit_app_shutdown (GApplication *app)
 {
 	gedit_debug_message (DEBUG_APP, "Quitting\n");
 
@@ -378,10 +959,16 @@ gedit_app_quit_impl (GeditApp *app)
 	ensure_user_config_dir ();
 
 	save_accels ();
-	save_page_setup (app);
-	save_print_settings (app);
+	save_page_setup (GEDIT_APP (app));
+	save_print_settings (GEDIT_APP (app));
 
-	gtk_main_quit ();
+#ifndef ENABLE_GVFS_METADATA
+	gedit_metadata_manager_shutdown ();
+#endif
+
+	gedit_dirs_shutdown ();
+
+	G_APPLICATION_CLASS (gedit_app_parent_class)->shutdown (app);
 }
 
 static void
@@ -403,29 +990,11 @@ load_accels (void)
 static void
 gedit_app_constructed (GObject *object)
 {
+	G_OBJECT_CLASS (gedit_app_parent_class)->constructed (object);
+
 	load_accels ();
 }
 
-static void
-set_active_window (GeditApp    *app,
-                   GeditWindow *window)
-{
-	app->priv->active_window = window;
-}
-
-static gboolean
-window_focus_in_event (GeditWindow   *window, 
-		       GdkEventFocus *event, 
-		       GeditApp      *app)
-{
-	/* updates active_view and active_child when a new toplevel receives focus */
-	g_return_val_if_fail (GEDIT_IS_WINDOW (window), FALSE);
-
-	set_active_window (app, window);
-
-	return FALSE;
-}
-
 static gboolean 
 window_delete_event (GeditWindow *window,
                      GdkEvent    *event,
@@ -436,9 +1005,7 @@ window_delete_event (GeditWindow *window,
 	ws = gedit_window_get_state (window);
 
 	if (ws & 
-	    (GEDIT_WINDOW_STATE_SAVING |
-	     GEDIT_WINDOW_STATE_PRINTING |
-	     GEDIT_WINDOW_STATE_SAVING_SESSION))
+	    (GEDIT_WINDOW_STATE_SAVING | GEDIT_WINDOW_STATE_PRINTING))
 	{
 		return TRUE;
 	}
@@ -449,111 +1016,45 @@ window_delete_event (GeditWindow *window,
 	return TRUE;
 }
 
-void
-_gedit_app_quit (GeditApp *app)
-{
-	GEDIT_APP_GET_CLASS (app)->quit (app);
-}
-
-static void
-window_destroy (GeditWindow *window, 
-		GeditApp    *app)
-{
-	app->priv->windows = g_list_remove (app->priv->windows,
-					    window);
-
-	if (window == app->priv->active_window)
-	{
-		set_active_window (app, app->priv->windows != NULL ? app->priv->windows->data : NULL);
-	}
-
-/* CHECK: I don't think we have to disconnect this function, since windows
-   is being destroyed */
-/*					     
-	g_signal_handlers_disconnect_by_func (window, 
-					      G_CALLBACK (window_focus_in_event),
-					      app);
-	g_signal_handlers_disconnect_by_func (window, 
-					      G_CALLBACK (window_destroy),
-					      app);
-*/
-	if (app->priv->windows == NULL)
-	{
-		if (!GEDIT_APP_GET_CLASS (app)->last_window_destroyed (app, window))
-		{
-			return;
-		}
-
-		_gedit_app_quit (app);
-	}
-}
-
 static GeditWindow *
 gedit_app_create_window_impl (GeditApp *app)
 {
 	GeditWindow *window;
-	gboolean isfirst;
 	
-	/*
-	 * We need to be careful here, there is a race condition:
-	 * when another gedit is launched it checks active_window,
-	 * so we must do our best to ensure that active_window
-	 * is never NULL when at least a window exists.
-	 */
+	window = g_object_new (GEDIT_TYPE_WINDOW, "application", app, NULL);
 
-	isfirst = (app->priv->windows == NULL);
-	
-	window = g_object_new (GEDIT_TYPE_WINDOW, NULL);
-
-	if (isfirst)
-	{
-		set_active_window (app, window);
-	}
-
-	app->priv->windows = g_list_prepend (app->priv->windows,
-					     window);
+	gtk_application_add_window (GTK_APPLICATION (app), GTK_WINDOW (window));
 
 	gedit_debug_message (DEBUG_APP, "Window created");
 
-	g_signal_connect (window, 
-			  "focus_in_event",
-			  G_CALLBACK (window_focus_in_event), 
-			  app);
 	g_signal_connect (window,
 			  "delete_event",
 			  G_CALLBACK (window_delete_event),
-			  app);			  
-	g_signal_connect (window, 
-			  "destroy",
-			  G_CALLBACK (window_destroy),
 			  app);
 
 	return window;
 }
 
-static void
-gedit_app_ready_impl (GeditApp *app)
-{
-}
-
 static void
 gedit_app_class_init (GeditAppClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
+	GApplicationClass *app_class = G_APPLICATION_CLASS (klass);
 
-	object_class->finalize = gedit_app_finalize;
 	object_class->dispose = gedit_app_dispose;
 	object_class->get_property = gedit_app_get_property;
-	object_class->constructor = gedit_app_constructor;
 	object_class->constructed = gedit_app_constructed;
 
-	klass->last_window_destroyed = gedit_app_last_window_destroyed_impl;
+	app_class->startup = gedit_app_startup;
+	app_class->activate = gedit_app_activate;
+	app_class->command_line = gedit_app_command_line;
+	app_class->local_command_line = gedit_app_local_command_line;
+	app_class->shutdown = gedit_app_shutdown;
+
 	klass->show_help = gedit_app_show_help_impl;
 	klass->help_link_id = gedit_app_help_link_id_impl;
 	klass->set_window_title = gedit_app_set_window_title_impl;
-	klass->quit = gedit_app_quit_impl;
 	klass->create_window = gedit_app_create_window_impl;
-	klass->ready = gedit_app_ready_impl;
 
 	g_object_class_install_property (object_class,
 					 PROP_LOCKDOWN,
@@ -630,96 +1131,10 @@ load_print_settings (GeditApp *app)
 		app->priv->print_settings = gtk_print_settings_new ();
 }
 
-static void
-extension_added (PeasExtensionSet *extensions,
-		 PeasPluginInfo   *info,
-		 PeasExtension    *exten,
-		 GeditApp         *app)
-{
-	gedit_app_activatable_activate (GEDIT_APP_ACTIVATABLE (exten));
-}
-
-static void
-extension_removed (PeasExtensionSet *extensions,
-		   PeasPluginInfo   *info,
-		   PeasExtension    *exten,
-		   GeditApp         *app)
-{
-	gedit_app_activatable_deactivate (GEDIT_APP_ACTIVATABLE (exten));
-}
-
 static void
 gedit_app_init (GeditApp *app)
 {
-	GtkSourceStyleSchemeManager *manager;
-
 	app->priv = GEDIT_APP_GET_PRIVATE (app);
-
-	/* Load settings */
-	app->priv->settings = gedit_settings_new ();
-	app->priv->window_settings = g_settings_new ("org.gnome.gedit.state.window");
-
-	/* initial lockdown state */
-	app->priv->lockdown = gedit_settings_get_lockdown (GEDIT_SETTINGS (app->priv->settings));
-
-	/*
-	 * We use the default gtksourceview style scheme manager so that plugins
-	 * can obtain it easily without a gedit specific api, but we need to
-	 * add our search path at startup before the manager is actually used.
-	 */
-	manager = gtk_source_style_scheme_manager_get_default ();
-	gtk_source_style_scheme_manager_append_search_path (manager,
-							    gedit_dirs_get_user_styles_dir ());
-
-	app->priv->extensions = peas_extension_set_new (PEAS_ENGINE (gedit_plugins_engine_get_default ()),
-							GEDIT_TYPE_APP_ACTIVATABLE,
-							"app", app,
-							NULL);
-
-	g_signal_connect (app->priv->extensions,
-			  "extension-added",
-			  G_CALLBACK (extension_added),
-			  app);
-
-	g_signal_connect (app->priv->extensions,
-			  "extension-removed",
-			  G_CALLBACK (extension_removed),
-			  app);
-
-	peas_extension_set_foreach (app->priv->extensions,
-	                            (PeasExtensionSetForeachFunc) extension_added,
-	                            app);
-}
-
-/**
- * gedit_app_get_default:
- *
- * Returns the #GeditApp object. This object is a singleton and
- * represents the running gedit instance.
- *
- * Return value: (transfer none): the #GeditApp pointer
- */
-GeditApp *
-gedit_app_get_default (void)
-{
-	GType type;
-
-	if (app_instance != NULL)
-	{
-		return app_instance;
-	}
-
-#ifdef OS_OSX
-	type = GEDIT_TYPE_APP_OSX;
-#else
-#ifdef G_OS_WIN32
-	type = GEDIT_TYPE_APP_WIN32;
-#else
-	type = GEDIT_TYPE_APP_X11;
-#endif
-#endif
-
-	return GEDIT_APP (g_object_new (type, NULL));
 }
 
 /* Generates a unique string for a window role */
@@ -738,64 +1153,6 @@ gen_role (void)
 				g_get_host_name ());
 }
 
-static GeditWindow *
-gedit_app_create_window_real (GeditApp    *app,
-			      gboolean     set_geometry,
-			      const gchar *role)
-{
-	GeditWindow *window;
-	
-	window = GEDIT_APP_GET_CLASS (app)->create_window (app);
-
-	if (role != NULL)
-	{
-		gtk_window_set_role (GTK_WINDOW (window), role);
-	}
-	else
-	{
-		gchar *newrole;
-
-		newrole = gen_role ();
-		gtk_window_set_role (GTK_WINDOW (window), newrole);
-		g_free (newrole);
-	}
-
-	if (set_geometry)
-	{
-		GdkWindowState state;
-		gint w, h;
-
-		state = g_settings_get_int (app->priv->window_settings,
-					    GEDIT_SETTINGS_WINDOW_STATE);
-
-		g_settings_get (app->priv->window_settings,
-				GEDIT_SETTINGS_WINDOW_SIZE,
-				"(ii)", &w, &h);
-
-		gtk_window_set_default_size (GTK_WINDOW (window), w, h);
-
-		if ((state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
-		{
-			gtk_window_maximize (GTK_WINDOW (window));
-		}
-		else
-		{
-			gtk_window_unmaximize (GTK_WINDOW (window));
-		}
-
-		if ((state & GDK_WINDOW_STATE_STICKY ) != 0)
-		{
-			gtk_window_stick (GTK_WINDOW (window));
-		}
-		else
-		{
-			gtk_window_unstick (GTK_WINDOW (window));
-		}
-	}
-
-	return window;
-}
-
 /**
  * gedit_app_create_window:
  * @app: the #GeditApp
@@ -810,178 +1167,51 @@ gedit_app_create_window (GeditApp  *app,
 			 GdkScreen *screen)
 {
 	GeditWindow *window;
+	gchar *role;
+	GdkWindowState state;
+	gint w, h;
 
 	gedit_debug (DEBUG_APP);
 
-	window = gedit_app_create_window_real (app, TRUE, NULL);
+	window = GEDIT_APP_GET_CLASS (app)->create_window (app);
 
 	if (screen != NULL)
 	{
 		gtk_window_set_screen (GTK_WINDOW (window), screen);
 	}
 
-	return window;
-}
+	role = gen_role ();
+	gtk_window_set_role (GTK_WINDOW (window), role);
+	g_free (role);
 
-/*
- * Same as _create_window, but doesn't set the geometry.
- * The session manager takes care of it. Used in gnome-session.
- */
-GeditWindow *
-_gedit_app_restore_window (GeditApp    *app,
-			   const gchar *role)
-{
-	GeditWindow *window;
+	state = g_settings_get_int (app->priv->window_settings,
+				    GEDIT_SETTINGS_WINDOW_STATE);
 
-	gedit_debug (DEBUG_APP);
+	g_settings_get (app->priv->window_settings,
+			GEDIT_SETTINGS_WINDOW_SIZE,
+			"(ii)", &w, &h);
 
-	window = gedit_app_create_window_real (app, FALSE, role);
+	gtk_window_set_default_size (GTK_WINDOW (window), w, h);
 
-	return window;
-}
-
-/**
- * gedit_app_get_windows:
- * @app: the #GeditApp
- *
- * Returns all the windows currently present in #GeditApp.
- *
- * Return value: (element-type Gedit.Window) (transfer none): the list of #GeditWindows objects.
- * The list should not be freed
- */
-const GList *
-gedit_app_get_windows (GeditApp *app)
-{
-	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
-
-	return app->priv->windows;
-}
-
-/**
- * gedit_app_get_active_window:
- * @app: the #GeditApp
- *
- * Retrives the #GeditWindow currently active.
- *
- * Return value: (transfer none): the active #GeditWindow
- */
-GeditWindow *
-gedit_app_get_active_window (GeditApp *app)
-{
-	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
-
-	/* make sure our active window is always realized:
-	 * this is needed on startup if we launch two gedit fast
-	 * enough that the second instance comes up before the
-	 * first one shows its window.
-	 */
-	if (!gtk_widget_get_realized (GTK_WIDGET (app->priv->active_window)))
-		gtk_widget_realize (GTK_WIDGET (app->priv->active_window));
-
-	return app->priv->active_window;
-}
-
-static gboolean
-is_in_viewport (GeditWindow  *window,
-		GdkScreen    *screen,
-		gint          workspace,
-		gint          viewport_x,
-		gint          viewport_y)
-{
-	GdkScreen *s;
-	GdkDisplay *display;
-	GdkWindow *gdkwindow;
-	const gchar *cur_name;
-	const gchar *name;
-	gint cur_n;
-	gint n;
-	gint ws;
-	gint sc_width, sc_height;
-	gint x, y, width, height;
-	gint vp_x, vp_y;
-
-	/* Check for screen and display match */
-	display = gdk_screen_get_display (screen);
-	cur_name = gdk_display_get_name (display);
-	cur_n = gdk_screen_get_number (screen);
-
-	s = gtk_window_get_screen (GTK_WINDOW (window));
-	display = gdk_screen_get_display (s);
-	name = gdk_display_get_name (display);
-	n = gdk_screen_get_number (s);
-
-	if (strcmp (cur_name, name) != 0 || cur_n != n)
-		return FALSE;
-
-	/* Check for workspace match */
-	ws = gedit_utils_get_window_workspace (GTK_WINDOW (window));
-	if (ws != workspace && ws != GEDIT_ALL_WORKSPACES)
-		return FALSE;
-
-	/* Check for viewport match */
-	gdkwindow = gtk_widget_get_window (GTK_WIDGET (window));
-	gdk_window_get_position (gdkwindow, &x, &y);
-	width = gdk_window_get_width (gdkwindow);
-	height = gdk_window_get_height (gdkwindow);
-	gedit_utils_get_current_viewport (screen, &vp_x, &vp_y);
-	x += vp_x;
-	y += vp_y;
-
-	sc_width = gdk_screen_get_width (screen);
-	sc_height = gdk_screen_get_height (screen);
-
-	return x + width * .25 >= viewport_x &&
-	       x + width * .75 <= viewport_x + sc_width &&
-	       y  >= viewport_y &&
-	       y + height <= viewport_y + sc_height;
-}
-
-/**
- * _gedit_app_get_window_in_viewport:
- * @app: the #GeditApp
- * @screen: the #GdkScreen
- * @workspace: the workspace number
- * @viewport_x: the viewport horizontal origin
- * @viewport_y: the viewport vertical origin
- *
- * Since a workspace can be larger than the screen, it is divided into several
- * equal parts called viewports. This function retrives the #GeditWindow in
- * the given viewport of the given workspace.
- *
- * Return value: the #GeditWindow in the given viewport of the given workspace
- *    or %NULL% if no window is found.
- */
-GeditWindow *
-_gedit_app_get_window_in_viewport (GeditApp  *app,
-				   GdkScreen *screen,
-				   gint       workspace,
-				   gint       viewport_x,
-				   gint       viewport_y)
-{
-	GeditWindow *window;
-
-	GList *l;
-
-	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
-
-	/* first try if the active window */
-	window = app->priv->active_window;
-
-	g_return_val_if_fail (GEDIT_IS_WINDOW (window), NULL);
-
-	if (is_in_viewport (window, screen, workspace, viewport_x, viewport_y))
-		return window;
-
-	/* otherwise try to see if there is a window on this workspace */
-	for (l = app->priv->windows; l != NULL; l = l->next)
+	if ((state & GDK_WINDOW_STATE_MAXIMIZED) != 0)
 	{
-		window = l->data;
-
-		if (is_in_viewport (window, screen, workspace, viewport_x, viewport_y))
-			return window;
+		gtk_window_maximize (GTK_WINDOW (window));
+	}
+	else
+	{
+		gtk_window_unmaximize (GTK_WINDOW (window));
 	}
 
-	return NULL;
+	if ((state & GDK_WINDOW_STATE_STICKY ) != 0)
+	{
+		gtk_window_stick (GTK_WINDOW (window));
+	}
+	else
+	{
+		gtk_window_unstick (GTK_WINDOW (window));
+	}
+
+	return window;
 }
 
 /**
@@ -997,18 +1227,15 @@ GList *
 gedit_app_get_documents	(GeditApp *app)
 {
 	GList *res = NULL;
-	GList *windows;
+	GList *windows, *l;
 
 	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
 
-	windows = app->priv->windows;
-
-	while (windows != NULL)
+	windows = gtk_application_get_windows (GTK_APPLICATION (app));
+	for (l = windows; l != NULL; l = g_list_next (l))
 	{
 		res = g_list_concat (res,
-				     gedit_window_get_documents (GEDIT_WINDOW (windows->data)));
-
-		windows = g_list_next (windows);
+				     gedit_window_get_documents (GEDIT_WINDOW (l->data)));
 	}
 
 	return res;
@@ -1027,18 +1254,15 @@ GList *
 gedit_app_get_views (GeditApp *app)
 {
 	GList *res = NULL;
-	GList *windows;
+	GList *windows, *l;
 
 	g_return_val_if_fail (GEDIT_IS_APP (app), NULL);
 
-	windows = app->priv->windows;
-
-	while (windows != NULL)
+	windows = gtk_application_get_windows (GTK_APPLICATION (app));
+	for (l = windows; l != NULL; l = g_list_next (l))
 	{
 		res = g_list_concat (res,
-				     gedit_window_get_views (GEDIT_WINDOW (windows->data)));
-
-		windows = g_list_next (windows);
+				     gedit_window_get_views (GEDIT_WINDOW (l->data)));
 	}
 
 	return res;
@@ -1102,11 +1326,14 @@ gedit_app_process_window_event (GeditApp    *app,
 static void
 app_lockdown_changed (GeditApp *app)
 {
-	GList *l;
+	GList *windows, *l;
 
-	for (l = app->priv->windows; l != NULL; l = l->next)
+	windows = gtk_application_get_windows (GTK_APPLICATION (app));
+	for (l = windows; l != NULL; l = g_list_next (l))
+	{
 		_gedit_window_set_lockdown (GEDIT_WINDOW (l->data),
-					    app->priv->lockdown);
+		                            app->priv->lockdown);
+	}
 
 	g_object_notify (G_OBJECT (app), "lockdown");
 }
@@ -1195,12 +1422,4 @@ _gedit_app_get_settings (GeditApp *app)
 	return app->priv->settings;
 }
 
-void
-_gedit_app_ready (GeditApp *app)
-{
-	g_return_if_fail (GEDIT_IS_APP (app));
-
-	GEDIT_APP_GET_CLASS (app)->ready (app);
-}
-
 /* ex:set ts=8 noet: */
