@@ -37,6 +37,7 @@
 #include "gedit-utils.h"
 #include "gedit-multi-notebook.h"
 #include "gedit-notebook.h"
+#include "gedit-notebook-popup-menu.h"
 #include "gedit-cell-renderer-button.h"
 
 #include <glib/gi18n.h>
@@ -617,9 +618,11 @@ treeview_selection_changed (GtkTreeSelection    *selection,
 		{
 			gedit_multi_notebook_set_active_tab (panel->priv->mnb,
 							     tab);
+			if (notebook != NULL)
+				g_object_unref (notebook);
 			g_object_unref (tab);
 		}
-		else
+		else if (notebook != NULL)
 		{
 			panel->priv->setting_active_notebook = TRUE;
 			gtk_widget_grab_focus (GTK_WIDGET (notebook));
@@ -633,6 +636,8 @@ treeview_selection_changed (GtkTreeSelection    *selection,
 						  G_CALLBACK (document_changed),
 						  panel);
 			}
+
+			g_object_unref (notebook);
 		}
 	}
 }
@@ -767,18 +772,16 @@ menu_position (GtkMenu             *menu,
 	w = panel->priv->treeview;
 
 	path = get_current_path (panel);
-
 	gtk_tree_view_get_cell_area (GTK_TREE_VIEW (w),
 				     path,
 				     NULL,
 				     &rect);
+	gtk_tree_path_free (path);
 
 	wy = rect.y;
 
 	gdk_window_get_origin (gtk_widget_get_window (w), x, y);
-
 	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
-
 	gtk_widget_get_allocation (w, &allocation);
 
 	if (gtk_widget_get_direction (w) == GTK_TEXT_DIR_RTL)
@@ -800,15 +803,14 @@ menu_position (GtkMenu             *menu,
 
 static gboolean
 show_tab_popup_menu (GeditDocumentsPanel *panel,
+		     GeditTab            *tab,
 		     GdkEventButton      *event)
 {
 	GtkWidget *menu;
 
 	gedit_debug (DEBUG_PANEL);
 
-	menu = gtk_ui_manager_get_widget (gedit_window_get_ui_manager (panel->priv->window),
-					  "/NotebookPopup");
-	g_return_val_if_fail (menu != NULL, FALSE);
+	menu = gedit_notebook_popup_menu_new (panel->priv->window, tab);
 
 	if (event != NULL)
 	{
@@ -845,8 +847,9 @@ panel_button_press_event (GtkTreeView         *treeview,
 
 	gedit_debug (DEBUG_PANEL);
 
-	if (event->type == GDK_BUTTON_PRESS && event->button == 3 &&
-	    event->window == gtk_tree_view_get_bin_window (treeview))
+	if ((event->type == GDK_BUTTON_PRESS) &&
+	    (gdk_event_triggers_context_menu ((GdkEvent *) event)) &&
+	    (event->window == gtk_tree_view_get_bin_window (treeview)))
 	{
 		GtkTreePath *path = NULL;
 
@@ -883,7 +886,7 @@ panel_button_press_event (GtkTreeView         *treeview,
 								  FALSE);
 
 					/* A row exists at the mouse position */
-					ret = show_tab_popup_menu (panel, event);
+					ret = show_tab_popup_menu (panel, tab, event);
 
 					g_object_unref (tab);
 				}
